
#include "../include/ARPluginHelper.h"

#define EFFECT_ID 4467
#define EFFECT_VERSION 1

#define EFFECT_EDITOR_WIDTH 200
#define EFFECT_EDITOR_HEIGHT 100

typedef struct {
	int leftMute;
	int rightMute;
} EffectState;

class ChannelMutePlugin : public ARPluginHelper, public ARCheckBoxListener
{
protected:
	ARCheckBox* chkLeft;
	ARCheckBox* chkRight;
	EffectState state;
	HWND effectWindow;

public:
	ChannelMutePlugin()
	{
		chkLeft = NULL;
		chkRight = NULL;
		effectWindow = 0;
		state.leftMute = R_FALSE;
		state.rightMute = R_FALSE;
	}

	virtual ~ChannelMutePlugin()
	{

	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"Channel Mute");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		effectWindow = parentWindow;

		interfaceAPI->CreateCheckBox(effectWindow, &chkLeft);
		interfaceAPI->CreateCheckBox(effectWindow, &chkRight);

		chkLeft->SetPosition(20, 20);
		chkRight->SetPosition(20, 50);

		chkLeft->SetText(RSTR(L"Mute Left"));
		chkRight->SetText(RSTR(L"Mute Right"));

		chkLeft->SetCheckedState(state.leftMute);
		chkRight->SetCheckedState(state.rightMute);

		chkLeft->SetListener(this);
		chkRight->SetListener(this);
	}

	void STDCALL DestroyEditor()
	{
		chkLeft->Release();
		chkRight->Release();

		chkLeft = NULL;
		chkRight = NULL;
	}

	void STDCALL LoadState(void *data, int size)
	{
		if (size == sizeof(EffectState))
		{
			state.leftMute = ((EffectState*)data)->leftMute;
			state.rightMute = ((EffectState*)data)->rightMute;

			if (chkLeft) // update only if created
			{
				chkLeft->SetCheckedState(state.leftMute);
				chkRight->SetCheckedState(state.rightMute);
			}
		}
	}

	int STDCALL SaveState(void **data)
	{
		*data = &state;
		return sizeof(EffectState);
	}

	void STDCALL OnCheckBoxStateChange(int checkBoxID)
	{
		if (checkBoxID == chkLeft->GetID())
			state.leftMute = chkLeft->IsChecked();
		else if (checkBoxID == chkRight->GetID())
			state.rightMute = chkRight->IsChecked();
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		for (int i = 0; i < frameCount; ++i)
		{
			outputs[0][i] = (state.leftMute == R_TRUE) ? 0 : inputs[0][i];
			outputs[1][i] = (state.rightMute == R_TRUE) ? 0 : inputs[1][i];
		}
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_EFFECT(ChannelMutePlugin)
