
#include "../include/ARPluginHelper.h"

// use your own id
#define EFFECT_ID 4455
#define EFFECT_VERSION 1

#define EFFECT_EDITOR_WIDTH 200
#define EFFECT_EDITOR_HEIGHT 100

typedef struct {
	int gain;
} EffectState;

class ExamplePlugin2 : public ARPluginHelper, public ARSliderListener
{
protected:
	ARSlider* slider;
	EffectState state;
	HWND effectWindow;

public:
	ExamplePlugin2()
	{
		slider = 0;
		effectWindow = 0;
		state.gain = 50;
	}

	virtual ~ExamplePlugin2()
	{

	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"Audio Gain");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		effectWindow = parentWindow;
		interfaceAPI->CreateSlider(parentWindow, R_FALSE, &slider);
		slider->SetPosition(40, 40);
		slider->SetValue(this->state.gain);
		slider->SetListener(this);
	}

	void STDCALL DestroyEditor()
	{
		slider->Release();
		slider = 0;
	}

	void STDCALL LoadState(void *data, int size)
	{
		if (size == sizeof(EffectState))
		{
			state.gain = ((EffectState*)data)->gain;

			if (slider) // update if created
				slider->SetValue(state.gain);
		}
	}

	int STDCALL SaveState(void **data)
	{
		*data = &state;
		return sizeof(EffectState);
	}

	void STDCALL OnSliderChange(int sliderID)
	{
		if (sliderID == slider->GetID())
		{
			state.gain = slider->GetValue();
		}
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		float mul = (state.gain / 100.0f) * 2;

		for (int i = 0; i < frameCount; ++i)
		{
			outputs[0][i] = mul * inputs[0][i];
			outputs[1][i] = mul * inputs[1][i];
		}
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_EFFECT(ExamplePlugin2)
