

#include "../include/ARPluginHelper.h"

// use your own id
#define EFFECT_ID 4456
#define EFFECT_VERSION 1

#define EFFECT_EDITOR_WIDTH 300
#define EFFECT_EDITOR_HEIGHT 220

class MyEditor : public CRUnknown, public ARButtonListener, public ARCheckBoxListener, public ARSliderListener
{
	ARButton* button;
	ARLabel* label;
	ARLabel* label2;
	ARLabel* headerLabel;
	ARCheckBox* checkBox;
	ARSlider* hSlider;
	ARSlider* vSlider;
	ARComboBox* comboBox;
	ARProgressBar* progressBar;

public:
	MyEditor()
	{
		button = 0;
		label = 0;
		checkBox = 0;
		hSlider = 0;
		vSlider = 0;
		comboBox = 0;
		progressBar = 0;
	}

	void CreateEditor(HWND parentWindow, UserInterfaceAPI* interfaceAPI)
	{	
		interfaceAPI->CreateButton(parentWindow, &button);
		button->SetPosition(10, 10);
		button->SetListener(this);
		button->SetText(RSTR(L"Button"));

		interfaceAPI->CreateLabel(parentWindow, R_FALSE, &label);
		label->SetPosition(10, 50);
		label->SetTextColor(RGB(251, 203, 73));
		label->SetText(RSTR(L"This is a Label"));

		interfaceAPI->CreateLabel(parentWindow, R_FALSE, &label2);
		label2->SetPosition(200, 150);
		label2->SetText(RSTR(L"Value: 50%"));

		interfaceAPI->CreateLabel(parentWindow, R_TRUE, &headerLabel);
		headerLabel->SetPosition(10, 80);
		headerLabel->SetTextColor(RGB(80, 255, 80));
		headerLabel->SetText(RSTR(L"This is a Header Label"));

		interfaceAPI->CreateCheckBox(parentWindow, &checkBox);
		checkBox->SetPosition(140, 10);
		checkBox->SetListener(this);
		checkBox->SetText(RSTR(L"This is a CheckBox"));

		interfaceAPI->CreateSlider(parentWindow, R_FALSE, &hSlider);
		hSlider->SetPosition(140, 50);
		hSlider->SetSize(110, 15);

		interfaceAPI->CreateSlider(parentWindow, R_TRUE, &vSlider);
		vSlider->SetPosition(170, 120);
		vSlider->SetSize(15, 80);
		vSlider->SetValue(50);
		vSlider->SetListener(this);

		interfaceAPI->CreateComboBox(parentWindow, &comboBox);
		comboBox->AddItem(RSTR(L"Item 1"));
		comboBox->AddItem(RSTR(L"Item 2"));
		comboBox->AddItem(RSTR(L"Item 3"));
		comboBox->AddItem(RSTR(L"Item 4"));
		comboBox->SetSelectedItem(0);
		comboBox->SetPosition(140, 80);

		interfaceAPI->CreateProgressBar(parentWindow, ARProgressBarStyles::FAT_BLOCKS, &progressBar);
		progressBar->SetPosition(10, 150);
		progressBar->SetSize(122, 15);
		progressBar->SetValue(50);
		progressBar->SetColor(RGB(119, 187, 255));
	}

	void DestroyEditor()
	{
		button->Release();
		label->Release();
		label2->Release();
		headerLabel->Release();
		checkBox->Release();
		hSlider->Release();
		vSlider->Release();
		comboBox->Release();
		progressBar->Release();
	}

	void STDCALL OnButtonPress(int buttonID)
	{
		if (buttonID == this->button->GetID())
			label->SetText(RSTR(L"button1 pressed"));
	}

	void STDCALL OnCheckBoxStateChange(int checkBoxID)
	{
		if (checkBoxID == this->checkBox->GetID())
			label->SetTextColor((checkBox->IsChecked() == R_TRUE) ? RGB(80, 80, 255) : RGB(251, 203, 73));
	}

	void STDCALL OnSliderChange(int sliderID)
	{
		if (sliderID == vSlider->GetID())
		{
			int value = vSlider->GetValue();

			wchar_t buff[32];
			wsprintfW(buff, L"Value: %d%%", value);
			label2->SetText(RSTR(buff));

			progressBar->SetValue(value);
		}
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

class ExamplePlugin3 : public ARPluginHelper
{
	MyEditor* editor;

public:

	ExamplePlugin3()
	{
		editor = NULL;
	}

	virtual ~ExamplePlugin3()
	{ 

	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"Example GUI");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		editor = new MyEditor();
		editor->CreateEditor(parentWindow, interfaceAPI);
	}

	void STDCALL DestroyEditor()
	{
		editor->DestroyEditor();
		editor->Release();
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		for (int i = 0; i < frameCount; ++i)
		{
			outputs[0][i] = inputs[0][i];
			outputs[1][i] = inputs[1][i];
		}
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};


EXPORT_EFFECT(ExamplePlugin3)