
// Example plugin which uses dialog as an editor

#include "../../include/ARPluginHelper.h"
#include "resource.h"
#include <Commctrl.h>
#include <math.h>

// use your own id
#define EFFECT_ID 4455
#define EFFECT_VERSION 1
#define EFFECT_EDITOR_WIDTH 258
#define EFFECT_EDITOR_HEIGHT 138

struct PluginState
{
	int amount; // 0 to 100
};

INT_PTR CALLBACK DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			{
				PluginState* pluginState = (PluginState*)lParam;

				::SetPropW(hDlg, L"PLUGIN_STATE", (HANDLE)pluginState); // save state object as window property! so, we can use it on slider change.

				HWND slider = ::GetDlgItem(hDlg, IDC_SLIDER1);
				::SendMessageW(slider, TBM_SETRANGE, TRUE, (LPARAM)MAKELONG(0, 100));			
				::SendMessageW(slider, TBM_SETPOS, TRUE, (LPARAM)pluginState->amount);

				return (INT_PTR)TRUE;
			}

		case WM_HSCROLL: // on slider change
			{
				PluginState* pluginState = (PluginState*)::GetPropW(hDlg, L"PLUGIN_STATE"); // get state object from window handle

				HWND slider = ::GetDlgItem(hDlg, IDC_SLIDER1);
				pluginState->amount = (int)::SendMessageW(slider, TBM_GETPOS, 0, 0);
				return FALSE;
			}

		case WM_DESTROY:
			{
				::RemoveProp(hDlg, L"PLUGIN_STATE");
				return FALSE;
			}

		default: return FALSE;
	}
}

class ExamplePlugin4 : public ARPluginHelper
{
	PluginState state;
	HWND pluginWindow;

public:

	ExamplePlugin4()
	{
		state.amount = 50;
		pluginWindow = 0;
	}

	virtual ~ExamplePlugin4()
	{

	}

	// this plugin does not depend on gui api.
	int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam)
	{
		return 1;
	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"Audio Gain");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		pluginWindow = ::CreateDialogParamW(hInstance, MAKEINTRESOURCEW(IDD_DIALOG1), parentWindow, DialogProc, (LPARAM)&state); // pass state object as a parameter
		::SetWindowPos(pluginWindow, 0, 0, 0, EFFECT_EDITOR_WIDTH, EFFECT_EDITOR_HEIGHT, SWP_NOZORDER);

		::ShowWindow(pluginWindow, SW_SHOW);
	}

	void STDCALL DestroyEditor()
	{
		::DestroyWindow(pluginWindow);
		pluginWindow = 0;
	}

	void STDCALL LoadState(void *data, int size)
	{
		if (size == sizeof(PluginState))
		{
			state.amount = ((PluginState*)data)->amount;

			if (pluginWindow) // update gui if created
			{
				HWND slider = ::GetDlgItem(pluginWindow, IDC_SLIDER1);
				::SendMessageW(slider, TBM_SETPOS, TRUE, (LPARAM)state.amount);
			}
		}
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		float mul = (state.amount / 100.0f) * 2;

		for (int i = 0; i < frameCount; ++i)
		{
			outputs[0][i] = mul * inputs[0][i];
			outputs[1][i] = mul * inputs[1][i];
		}
	}

	int STDCALL SaveState(void **data)
	{
		*data = &state;
		return sizeof(PluginState);
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};


EXPORT_EFFECT(ExamplePlugin4)