
#include "../include/ARPluginHelper.h"
#include <math.h>

// use your own id
#define EFFECT_ID 4458
#define EFFECT_VERSION 1

#define EFFECT_EDITOR_WIDTH 200
#define EFFECT_EDITOR_HEIGHT 100

typedef struct {
	int gain;
} EffectState;

class ExamplePlugin : public ARPluginHelper, public ARSliderListener
{
protected:
	ARSlider* slider;
	EffectState state;
	HWND effectWindow;

public:
	ExamplePlugin()
	{
		slider = NULL;
		effectWindow = 0;
		state.gain = 0;
	}

	virtual ~ExamplePlugin()
	{

	}

	void STDCALL GetEffectName(RString** ppText)
	{
		*ppText = MakeRString(L"Stereo Width");
	}

	int STDCALL GetEffectID()
	{
		return EFFECT_ID;
	}

	int STDCALL GetEffectVersion()
	{
		return EFFECT_VERSION;
	}

	void STDCALL GetEditorSize(int *width, int *height)
	{
		*width = EFFECT_EDITOR_WIDTH;
		*height = EFFECT_EDITOR_HEIGHT;
	}

	void STDCALL CreateEditor(HWND parentWindow)
	{
		effectWindow = parentWindow;
		interfaceAPI->CreateSlider(parentWindow, R_FALSE, &slider);
		slider->SetPosition(40, 40);
		slider->SetValue(this->state.gain);
		slider->SetListener(this);
	}

	void STDCALL DestroyEditor()
	{
		slider->Release();
		slider = NULL;
	}

	void STDCALL LoadState(void *data, int size)
	{
		if (size == sizeof(EffectState))
		{
			state.gain = ((EffectState*)data)->gain;

			if (slider) // update if created
				slider->SetValue(state.gain);
		}
	}

	int STDCALL SaveState(void **data)
	{
		*data = &state;
		return sizeof(EffectState);
	}

	void STDCALL OnSliderChange(int sliderID)
	{
		if (sliderID == slider->GetID())
		{
			state.gain = slider->GetValue();
		}
	}

	void STDCALL ProcessAudio(float **inputs, float **outputs, long frameCount)
	{
		// code: https://www.kvraudio.com/forum/viewtopic.php?t=212587

		const float width = (state.gain / 100.0f) * 2;

		for (int i = 0; i < frameCount; ++i)
		{
			float L = inputs[0][i];
			float R = inputs[1][i];

			float M = (L + R) / ::sqrt(2.0f);   // obtain mid-signal from left and right
			float S = (L - R) / ::sqrt(2.0f);   // obtain side-signal from left and right

			// amplify mid and side signal seperately:
			M *= 2 * (1 - width);
			S *= 2 * width;

			L = (M + S) / ::sqrt(2.0f);   // obtain left signal from mid and side
			R = (M - S) / ::sqrt(2.0f);   // obtain right signal from mid and side

			outputs[0][i] = L;
			outputs[1][i] = R;
		}
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_EFFECT(ExamplePlugin)
