
// Audio Repeater Pro API Objects.
// (c) 2020 CrownSoft

#ifndef _AR_OBJECTS_H_
#define _AR_OBJECTS_H_

#include "RAudioEffect.h"

#define AUDIO_REPEATER_PRO_HOST_ID 9980

// simple API for prototyping user interfaces.
#define AUDIO_REPEATER_PRO_GUI_API 1001

class ARFactory : public RUnknown
{
public:
	// returns R_OK if object is available
	virtual int STDCALL GetAPIObject(int objectID, void** ppObject) = 0;
};

class ARComponent : public RUnknown
{
public:
	virtual void STDCALL SetText(RString* text) = 0;
	virtual void STDCALL SetSize(int width, int height) = 0;
	virtual void STDCALL SetPosition(int x, int y) = 0;

	// pass R_FALSE to make it hide 
	virtual void STDCALL SetVisible(R_BOOL visible) = 0;

	// pass R_FALSE to make it disable
	virtual void STDCALL SetEnable(R_BOOL enable) = 0;

	virtual void STDCALL GetText(RString** ppText) = 0;
	virtual int STDCALL GetWidth() = 0;
	virtual int STDCALL GetHeight() = 0;
	virtual int STDCALL GetPosX() = 0;
	virtual int STDCALL GetPosY() = 0;

	// returns unique identifier of this component
	virtual int STDCALL GetID() = 0;

	// returns R_TRUE if visible
	virtual R_BOOL STDCALL IsVisible() = 0;

	// returns R_TRUE if enabled
	virtual R_BOOL STDCALL IsEnabled() = 0;
};

class ARButtonListener : public RUnknown
{
public:
	virtual void STDCALL OnButtonPress(int buttonID) = 0;
};

class ARButton : public ARComponent
{
public:
	virtual void STDCALL SetListener(ARButtonListener* listener) = 0;
};

class ARLabel : public ARComponent
{
public:
	virtual void STDCALL SetTextColor(COLORREF textColor) = 0;

	// pass R_TRUE to auto resize
	virtual void STDCALL EnableAutoSize(R_BOOL enable) = 0;
};

class ARCheckBoxListener : public RUnknown
{
public:
	virtual void STDCALL OnCheckBoxStateChange(int checkBoxID) = 0;
};

class ARCheckBox : public ARComponent
{
public:
	virtual void STDCALL SetListener(ARCheckBoxListener* listener) = 0;

	// returns R_TRUE if checked
	virtual R_BOOL STDCALL IsChecked() = 0;

	// pass R_FALSE to uncheck
	virtual void STDCALL SetCheckedState(R_BOOL state) = 0;

	// pass R_TRUE to auto resize
	virtual void STDCALL EnableAutoSize(R_BOOL enable) = 0;
};

class ARSliderListener : public RUnknown
{
public:
	virtual void STDCALL OnSliderChange(int sliderID) = 0;
};

class ARSlider : public ARComponent
{
public:
	virtual void STDCALL SetListener(ARSliderListener* listener) = 0;
	virtual void STDCALL SetValue(int value) = 0;
	virtual int STDCALL GetValue() = 0;
};

class ARComboBoxListener : public RUnknown
{
public:
	virtual void STDCALL OnComboBoxItemSelect(int comboBoxID) = 0;
};

class ARComboBox : public ARComponent
{
public:
	virtual void STDCALL AddItem(RString* text) = 0;
	virtual void STDCALL UpdateItem(int index, RString* text) = 0;
	virtual void STDCALL GetItemText(int index, RString** ppText) = 0;
	virtual void STDCALL RemoveItem(int index) = 0;
	virtual void STDCALL RemoveAll() = 0;
	virtual int STDCALL GetSelectedItemIndex() = 0;
	virtual void STDCALL SetSelectedItem(int index) = 0;
	virtual int STDCALL GetItemCount() = 0;
	virtual void STDCALL SetListener(ARComboBoxListener* listener) = 0;
};

namespace ARProgressBarStyles
{
	enum ARProgressBarStyles
	{
		FAT_BLOCKS = 0,
		SLIM_BLOCKS = 1,
		FILL_BAR = 2
	};
};

class ARProgressBar : public ARComponent
{
public:
	virtual void STDCALL SetValue(int value) = 0;
	virtual int STDCALL GetValue() = 0;
	virtual void STDCALL SetColor(COLORREF color) = 0;
};
 
class UserInterfaceAPI : public RUnknown
{
public:
	// quick way to prototype a plugin
	virtual void STDCALL CreateButton(HWND parent, ARButton** ppButton) = 0;
	virtual void STDCALL CreateLabel(HWND parent, R_BOOL isHeader, ARLabel** ppLabel) = 0;
	virtual void STDCALL CreateCheckBox(HWND parent, ARCheckBox** ppCheckBox) = 0;
	virtual void STDCALL CreateSlider(HWND parent, R_BOOL isVertical, ARSlider** ppSlider) = 0;
	virtual void STDCALL CreateComboBox(HWND parent, ARComboBox** ppComboBox) = 0;
	virtual void STDCALL CreateProgressBar(HWND parent, int style, ARProgressBar** ppProgressBar) = 0;

	virtual HWND STDCALL GetPluginWindow(HWND containerWnd) = 0;
	virtual HWND STDCALL GetApplicationWindow() = 0;
	virtual void STDCALL SetContainerSize(HWND containerWnd, int width, int height) = 0;
	virtual void STDCALL UpdatePluginName() = 0;
};

#endif