
// COM compatible audio effect class which is intended to be used with CrownSoft products.
// (c) 2018 CrownSoft

#ifndef _RAUDIO_EFFECT_H_
#define _RAUDIO_EFFECT_H_

#include "RObjBase.h"

#define AE_PLUGIN_EXT L".ae"

class RAudioEffect : public RUnknown
{
public:

	// returns 0 on failure.
	virtual int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam) = 0;

	virtual void STDCALL GetEffectName(RString** ppText) = 0;

	// effect id is not necessary to be unique.
	virtual int STDCALL GetEffectID() = 0;

	virtual int STDCALL GetEffectVersion() = 0;

	// returns 0 if not supported.
	virtual int STDCALL SetChannelCount(int inputChannelCount, int outputChannelCount) = 0;

	virtual void STDCALL SetSampleRate(float srate) = 0;

	virtual void STDCALL SetBlockSize(int blockSize) = 0;

	virtual void STDCALL CreateEditor(HWND containerWnd) = 0;

	virtual void STDCALL GetEditorSize(int* width, int* height) = 0;

	virtual void STDCALL DestroyEditor() = 0;

	// param: size in bytes
	// host may call this method before or after calling the CreateEditor method.
	virtual void STDCALL LoadState(void* data, int size) = 0;

	// host may call this method before or after calling the CreateEditor method.
	virtual void STDCALL StartProcess() = 0;

	// called from audio thread.
	virtual void STDCALL ProcessAudio(float** inputs, float** outputs, long frameCount) = 0;

	// host may call this method before or after calling the DestroyEditor method.
	virtual void STDCALL StopProcess() = 0;

	// return value is the size of the memory in bytes.
	// host may call this method before or after calling the DestroyEditor method.
	virtual int STDCALL SaveState(void** data) = 0;

	// free resources
	virtual void STDCALL Shutdown() = 0;
};

typedef void (STDCALL *GetEffectFuncPtr)(RAudioEffect**);

#define EXPORT_EFFECT(className) extern "C" __declspec(dllexport) void STDCALL GetEffect(RAudioEffect** efffect) \
{ \
	*efffect = static_cast<RAudioEffect*>(new className()); \
} 

#endif