

#include "../include/ARObjects.h"

// use your own id
#define EFFECT_ID 4455
#define EFFECT_VERSION 1

typedef struct {
	int gain;
} EffectState;

typedef struct {
	RAudioEffectVtbl* pVtblRAudioEffect;
	ARSliderListenerVtbl* pVtblARSliderListener;

	UserInterfaceAPI* interfaceAPI;
	ARSlider* slider;
	HWND effectWindow;
	EffectState state;
	//=====================================
	long refCount;
	//=====================================

} ExampleEffect1;

IMPLEMENT_RUNKNOWN_FUNCTIONS(ExampleEffect1, RAudioEffectVtbl)
REDIRECT_RUNKNOWN_FUNCTIONS(ExampleEffect1, ARSliderListenerVtbl, pVtblRAudioEffect)

int STDCALL RAudioEffectVtbl_Initialize(void* _this, HINSTANCE hInstance, int hostID, void* hostParam)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);

	this->effectWindow = 0;
	this->state.gain = 50;

	if (hostID == AUDIO_REPEATER_PRO_HOST_ID)
	{
		ARFactory* factory = (ARFactory*)hostParam;
		if ((*factory)->GetAPIObject(factory, AUDIO_REPEATER_PRO_GUI_API, (void**)&this->interfaceAPI) == R_OK)
			return 1;
	}

	return 0;
}

void STDCALL RAudioEffectVtbl_GetEffectName(void* _this, RString** ppText)
{
	*ppText = MakeRString(L"Audio Gain");
}

int STDCALL RAudioEffectVtbl_GetEffectID(void* _this)
{
	return EFFECT_ID;
}

int STDCALL RAudioEffectVtbl_GetEffectVersion(void* _this)
{
	return EFFECT_VERSION;
}

int STDCALL RAudioEffectVtbl_SetChannelCount(void* _this, int inputChannelCount, int outputChannelCount)
{
	if ( (inputChannelCount == 2) && (outputChannelCount == 2) )
		return 1;

	return 0;
}

void STDCALL RAudioEffectVtbl_SetSampleRate(void* _this, float srate)
{

}

void STDCALL RAudioEffectVtbl_SetBlockSize(void* _this, int blockSize)
{

}

void STDCALL RAudioEffectVtbl_GetEditorSize(void* _this, int* width, int* height)
{
	*width = 200;
	*height = 100;
}

void STDCALL RAudioEffectVtbl_CreateEditor(void* _this, HWND container)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);

	this->effectWindow = container;

	(*this->interfaceAPI)->CreateSlider(this->interfaceAPI, container, R_FALSE, &this->slider);
	(*this->slider)->SetPosition(this->slider, 40, 40);
	(*this->slider)->SetValue(this->slider, this->state.gain);
	(*this->slider)->SetListener(this->slider, &this->pVtblARSliderListener);
}

void STDCALL RAudioEffectVtbl_DestroyEditor(void* _this)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);
	(*this->slider)->Release(this->slider);

	this->effectWindow = 0;
}

void STDCALL RAudioEffectVtbl_LoadState(void* _this, void* data, int size)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);

	if (size == sizeof(EffectState))
	{
		this->state.gain = ((EffectState*)data)->gain;

		if (this->effectWindow) // update gui if created
		{
			(*this->slider)->SetValue(this->slider, this->state.gain);
		}
	}
}

void STDCALL RAudioEffectVtbl_StartProcess(void* _this)
{

}

void STDCALL RAudioEffectVtbl_ProcessAudio(void* _this, float** inputs, float** outputs, long frameCount)
{
	int i;

	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);

	for (i = 0; i < frameCount; ++i)
	{
		float mul = (this->state.gain / 100.0f) * 2;
		outputs[0][i] = mul * inputs[0][i];
		outputs[1][i] = mul * inputs[1][i];
	}
}

void STDCALL RAudioEffectVtbl_StopProcess(void* _this)
{

}

int STDCALL RAudioEffectVtbl_SaveState(void* _this, void** data)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, RAudioEffectVtbl, _this);

	*data = &this->state;
	return sizeof(EffectState);
}

void STDCALL RAudioEffectVtbl_Shutdown(void* _this)
{

}

void STDCALL ARSliderListenerVtbl_OnSliderChange(void* _this, int sliderID)
{
	ExampleEffect1* this = GET_IMPL_PTR(ExampleEffect1, ARSliderListenerVtbl, _this);

	if (sliderID == (*this->slider)->GetID(this->slider))
	{
		this->state.gain = (*this->slider)->GetValue(this->slider);
	}
}

// constructor
ExampleEffect1* ExampleEffect1_Create()
{
	//=====================================================================
	ExampleEffect1* this = NEW_OBJ(ExampleEffect1);

	this->pVtblRAudioEffect = NEW_VTABLE(RAudioEffectVtbl);
	this->pVtblRAudioEffect->this_ = this;

	this->pVtblARSliderListener = NEW_VTABLE(ARSliderListenerVtbl);
	this->pVtblARSliderListener->this_ = this;

	VTABLE_FILL_RUNKNOWN_METHODS(RAudioEffectVtbl, this->pVtblRAudioEffect);
	VTABLE_FILL_RAUDIO_EFFECT_METHODS(RAudioEffectVtbl, this->pVtblRAudioEffect);

	VTABLE_FILL_RUNKNOWN_METHODS(ARSliderListenerVtbl, this->pVtblARSliderListener);
	VTABLE_FILL_METHOD(ARSliderListenerVtbl, this->pVtblARSliderListener, OnSliderChange);
	//=====================================================================

	this->refCount = 1;
	return this;
}

// destructor
void ExampleEffect1_Destroy(ExampleEffect1* this)
{

	//=====================================================================
	FREE_VTABLE(this->pVtblRAudioEffect);
	FREE_OBJ(this);
	//=====================================================================
}


EXPORT_EFFECT(ExampleEffect1, pVtblRAudioEffect)