
// API objects which can be used to do various tasks.
// (c) 2020 CrownSoft

#ifndef _R_OBJECTS_H_
#define _R_OBJECTS_H_

#include "RAudioEffect.h"

#define AUDIO_REPEATER_PRO_HOST_ID 9980

// simple API for prototyping user interfaces.
#define AUDIO_REPEATER_PRO_GUI_API 1001

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	int (STDCALL *GetAPIObject) (void* _this, int objectID, void** ppObject);

} ARFactoryVtbl;

typedef ARFactoryVtbl* ARFactory;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *OnButtonPress) (void* _this, int buttonID);

	//=====================================
	void* this_;
	//=====================================
} ARButtonListenerVtbl;

typedef ARButtonListenerVtbl* ARButtonListener;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *SetListener) (void* _this, ARButtonListener* listener);
} ARButtonVtbl;

typedef ARButtonVtbl* ARButton;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *SetTextColor) (void* _this, COLORREF textColor);
	void (STDCALL *EnableAutoSize) (void* _this, R_BOOL enable);
} ARLabelVtbl;

typedef ARLabelVtbl* ARLabel;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *OnCheckBoxStateChange) (void* _this, int checkBoxID);

	//=====================================
	void* this_;
	//=====================================
} ARCheckBoxListenerVtbl;

typedef ARCheckBoxListenerVtbl* ARCheckBoxListener;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *SetListener) (void* _this, ARCheckBoxListener* listener);
	R_BOOL (STDCALL *IsChecked) (void* _this);
	void (STDCALL *SetCheckedState) (void* _this, R_BOOL state);
	void (STDCALL *EnableAutoSize) (void* _this, R_BOOL enable);
} ARCheckBoxVtbl;

typedef ARCheckBoxVtbl* ARCheckBox;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *OnSliderChange) (void* _this, int sliderID);

	//=====================================
	void* this_;
	//=====================================
} ARSliderListenerVtbl;

typedef ARSliderListenerVtbl* ARSliderListener;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *SetListener) (void* _this, ARSliderListener* listener);
	void (STDCALL *SetValue) (void* _this, int value);
	int (STDCALL *GetValue) (void* _this);
} ARSliderVtbl;

typedef ARSliderVtbl* ARSlider;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *OnComboBoxItemSelect) (void* _this, int comboBoxID);

	//=====================================
	void* this_;
	//=====================================
} ARComboBoxListenerVtbl;

typedef ARComboBoxListenerVtbl* ARComboBoxListener;

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *AddItem) (void* _this, RString* text);
	void (STDCALL *UpdateItem) (void* _this, int index, RString* text);
	void (STDCALL *GetItemText) (void* _this, int index, RString** ppText);
	void (STDCALL *RemoveItem) (void* _this, int index);
	void (STDCALL *RemoveAll) (void* _this);
	int (STDCALL *GetSelectedItemIndex) (void* _this);
	void (STDCALL *SetSelectedItem) (void* _this, int index);
	int (STDCALL *GetItemCount) (void* _this);
	void (STDCALL *SetListener) (void* _this, ARComboBoxListener* listener);
} ARComboBoxVtbl;

typedef ARComboBoxVtbl* ARComboBox;

enum ARProgressBarStyles
{
	FAT_BLOCKS = 0,
	SLIM_BLOCKS = 1,
	FILL_BAR = 2
};

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *SetText) (void* _this, RString* text);
	void (STDCALL *SetSize) (void* _this, int width, int height);
	void (STDCALL *SetPosition) (void* _this, int x, int y);
	void (STDCALL *SetVisible) (void* _this, R_BOOL visible);
	void (STDCALL *SetEnable) (void* _this, R_BOOL enable);

	void (STDCALL *GetText) (void* _this, RString** ppText);
	int (STDCALL *GetWidth) (void* _this);
	int (STDCALL *GetHeight) (void* _this);
	int (STDCALL *GetPosX) (void* _this);
	int (STDCALL *GetPosY) (void* _this);
	int (STDCALL *GetID) (void* _this);

	R_BOOL (STDCALL *IsVisible) (void* _this);
	R_BOOL (STDCALL *IsEnabled) (void* _this);

	void (STDCALL *SetValue) (void* _this, int value);
	int (STDCALL *GetValue) (void* _this);
	void (STDCALL *SetColor) (void* _this, COLORREF color);
} ARProgressBarVtbl;

typedef ARProgressBarVtbl* ARProgressBar;

// quick way to prototype a plugin 
typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *CreateButton) (void* _this, HWND parent, ARButton** ppButton);
	void (STDCALL *CreateLabel) (void* _this, HWND parent, R_BOOL isHeader, ARLabel** ppLabel);
	void (STDCALL *CreateCheckBox) (void* _this, HWND parent, ARCheckBox** ppCheckBox);
	void (STDCALL *CreateSlider) (void* _this, HWND parent, R_BOOL isVertical, ARSlider** ppSlider);
	void (STDCALL *CreateComboBox) (void* _this, HWND parent, ARComboBox** ppComboBox);
	void (STDCALL *CreateProgressBar) (void* _this, HWND parent, int style, ARProgressBar** ppProgressBar);
	HWND (STDCALL *GetPluginWindow) (HWND containerWnd);
	HWND (STDCALL *GetApplicationWindow)();
	void (STDCALL *SetContainerSize) (HWND containerWnd, int width, int height);
	void (STDCALL *UpdatePluginName) ();
} UserInterfaceAPIVtbl;

typedef UserInterfaceAPIVtbl* UserInterfaceAPI;

#endif