
// (c) 2018 CrownSoft

#ifndef _AR_REMOTE_H_
#define _AR_REMOTE_H_

#include <windows.h>

const wchar_t* const AR_MAIN_WINDOW_CLASS_32 = L"Audio Repeater Pro Class - 32";
const wchar_t* const AR_MAIN_WINDOW_CLASS_64 = L"Audio Repeater Pro Class - 64";
const wchar_t* const AR_MAIN_WINDOW_TEXT = L"Audio Repeater Pro";


enum IPCMessages
{
	LOAD_FX_CHAIN = 6678,
	SET_BYPASS_FX_ENABLE = 6679,
	SET_PROCESSING_ENABLE = 6680,
	SET_STREAM_GAIN = 6681,
	REPEATER_START = 6682,
	REPEATER_STOP = 6683,
	IS_BYPASS_FX_ENABLED = 6684,
	IS_PROCESSING_ENABLED = 6685,
	GET_STREAM_GAIN = 6686,
	IS_REPEATER_STARTED = 6687
};

HWND GetRepeaterWindow(int is32Bit)
{
	return FindWindowW((is32Bit == 1) ? AR_MAIN_WINDOW_CLASS_32 : AR_MAIN_WINDOW_CLASS_64, AR_MAIN_WINDOW_TEXT);
}

void LoadFxChain(HWND windowHandle, wchar_t* fileName)
{
	COPYDATASTRUCT cds;
	cds.dwData = LOAD_FX_CHAIN;
	cds.cbData = (DWORD)(sizeof(wchar_t) * (wcslen(fileName) + 1));
	cds.lpData = fileName;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

void SetEnableFxBypass(HWND windowHandle, int enable)
{
	COPYDATASTRUCT cds;
	cds.dwData = SET_BYPASS_FX_ENABLE;
	cds.cbData = sizeof(int);
	cds.lpData = &enable;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

void SetEnableProcessing(HWND windowHandle, int enable)
{
	COPYDATASTRUCT cds;
	cds.dwData = SET_PROCESSING_ENABLE;
	cds.cbData = sizeof(int);
	cds.lpData = &enable;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

void SetStreamGain(HWND windowHandle, int gain)
{
	COPYDATASTRUCT cds;
	cds.dwData = SET_STREAM_GAIN;
	cds.cbData = sizeof(int);
	cds.lpData = &gain;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

void StartRepeater(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = REPEATER_START;
	cds.cbData = 0;
	cds.lpData = NULL;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

void StopRepeater(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = REPEATER_STOP;
	cds.cbData = 0;
	cds.lpData = NULL;

	SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

int IsBypassFxEnabled(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = IS_BYPASS_FX_ENABLED;
	cds.cbData = 0;
	cds.lpData = NULL;

	return SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

int IsProcessingEnabled(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = IS_PROCESSING_ENABLED;
	cds.cbData = 0;
	cds.lpData = NULL;

	return SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

int IsRepeaterStarted(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = IS_REPEATER_STARTED;
	cds.cbData = 0;
	cds.lpData = NULL;

	return SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

int GetStreamGain(HWND windowHandle)
{
	COPYDATASTRUCT cds;
	cds.dwData = GET_STREAM_GAIN;
	cds.cbData = 0;
	cds.lpData = NULL;

	return SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
}

#endif