
// Structs which are binary compatible with COM.
// (c) 2018 CrownSoft

#ifndef _R_OBJBASE_H_
#define _R_OBJBASE_H_

#ifndef _CRT_SECURE_NO_WARNINGS
	#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdlib.h>
#include <windows.h>

// types
#define R_BOOL int

// status values
#define R_OK 0
#define R_ERROR 1
#define R_NOINTERFACE 0x80004002L

// boolean values
#define R_TRUE 1
#define R_FALSE 0

#ifndef STDCALL
	#define STDCALL __stdcall
#endif

typedef struct _RGUID {
	unsigned int Data1;
	unsigned short Data2;
	unsigned short Data3;
	unsigned char Data4[8];
} RGUID;


#define GET_IMPL_PTR(implStructName, vtblName, ppVtbl) (implStructName*) (*(vtblName**)ppVtbl)->this_

#define IMPLEMENT_RUNKNOWN_FUNCTIONS(implStructName, vtblName) \
long STDCALL vtblName##_QueryInterface(void* _this, const RGUID* guid, void** ppv) \
	{ return R_NOINTERFACE; } \
unsigned long STDCALL vtblName##_AddRef(void* _this) \
	{ \
		implStructName* this = (implStructName*) (*(vtblName**)_this)->this_; \
		return InterlockedIncrement(&this->refCount); \
	} \
void implStructName##_Destroy(implStructName* this); \
unsigned long STDCALL vtblName##_Release(void* _this) \
	{ \
		implStructName* this = (implStructName*) (*(vtblName**)_this)->this_; \
		long res = InterlockedDecrement(&this->refCount); \
		if (res == 0) \
			implStructName##_Destroy(this); \
		return res; \
	}

#define REDIRECT_RUNKNOWN_FUNCTIONS(implStructName, vtblName, pImplVtbl) \
long STDCALL vtblName##_QueryInterface(void* _this, const RGUID* guid, void** ppv) \
	{ return R_NOINTERFACE; } \
unsigned long STDCALL vtblName##_AddRef(void* _this) \
	{ \
		implStructName* this = (implStructName*)(*(vtblName**)_this)->this_; \
		return this->pImplVtbl->AddRef(&this->pImplVtbl); \
	} \
unsigned long STDCALL vtblName##_Release(void* _this) \
{ \
	implStructName* this = (implStructName*)(*(vtblName**)_this)->this_; \
	return this->pImplVtbl->Release(&this->pImplVtbl); \
}

#define VTABLE_FILL_RUNKNOWN_METHODS(vtblName, pVtbl) pVtbl->QueryInterface = &vtblName##_QueryInterface; \
	pVtbl->AddRef = &vtblName##_AddRef; \
	pVtbl->Release = &vtblName##_Release;

#define VTABLE_FILL_METHOD(vtblName, ptrVtable, methodName) ptrVtable->methodName = &vtblName##_##methodName;

#define DECLARE_RUNKNOWN_VTABLE_ENTRIES long (STDCALL *QueryInterface) (void*, const RGUID*, void**); \
unsigned long (STDCALL *AddRef) (void*); \
unsigned long (STDCALL *Release) (void*);

#define NEW_OBJ(structName) (structName*)malloc(sizeof(structName))
#define NEW_VTABLE NEW_OBJ

#define FREE_OBJ(pStruct) free(pStruct)
#define FREE_VTABLE(pVtbl) free(pVtbl)

typedef struct {
	//=====================================
	DECLARE_RUNKNOWN_VTABLE_ENTRIES
	//=====================================

	void (STDCALL *GetString) (void*, const wchar_t **);
	int (STDCALL *GetLength) (void*);

	//=====================================
	void* this_;
	//=====================================
} RStringVtbl;

#define RString RStringVtbl*

typedef struct {
	RStringVtbl* pVtblRString;

	//=====================================
	long refCount;
	//=====================================

	wchar_t* text;
	int length;
} CRString;

IMPLEMENT_RUNKNOWN_FUNCTIONS(CRString, RStringVtbl)

void STDCALL RStringVtbl_GetString(void* _this, const wchar_t** ppText)
{
	//=====================================================================
	CRString* this = GET_IMPL_PTR(CRString, RStringVtbl, _this);
	//=====================================================================

	*ppText = this->text;
}

int STDCALL RStringVtbl_GetLength(void* _this)
{
	//=====================================================================
	CRString* this = GET_IMPL_PTR(CRString, RStringVtbl, _this);
	//=====================================================================

	return this->length;
}

// constructor
CRString* CRString_Create(const wchar_t* text)
{
	//=====================================================================
	CRString* this = NEW_OBJ(CRString);
	this->pVtblRString = NEW_VTABLE(RStringVtbl);
	this->pVtblRString->this_ = this;

	VTABLE_FILL_RUNKNOWN_METHODS(RStringVtbl, this->pVtblRString);
	//=====================================================================

	VTABLE_FILL_METHOD(RStringVtbl, this->pVtblRString, GetLength);
	VTABLE_FILL_METHOD(RStringVtbl, this->pVtblRString, GetString);

	this->refCount = 1;

	int len = wcslen(text);
	if (len)
	{
		this->text = (wchar_t*)malloc((len + 1) * sizeof(wchar_t));
		this->length = len;
		wcscpy(this->text, text);
	}
	else
	{
		if (text == 0) // null string
		{
			this->text = 0;
			this->length = 0;
		}
		else // empty string
		{
			this->text = (wchar_t*)malloc(sizeof(wchar_t));
			*this->text = 0;
			this->length = len;
		}
	}

	return this;
}

// destructor
void CRString_Destroy(CRString* this)
{
	if (this->text)
		free(this->text);

	//=====================================================================
	FREE_VTABLE(this->pVtblRString);
	FREE_OBJ(this);
	//=====================================================================
}

RString* MakeRString(const wchar_t* text)
{
	CRString* str = CRString_Create(text);
	return &str->pVtblRString;
}

#endif