
// API objects which can be used to do various tasks.
// (c) 2020 CrownSoft

unit ARObjectsUnit;

interface

uses
  Windows,
  RObjBaseUnit;

const
  AUDIO_REPEATER_PRO_HOST_ID = 9980;

  // simple API for prototyping user interfaces.
  AUDIO_REPEATER_PRO_GUI_API = 1001;

  // ProgressBar styles
  FAT_BLOCKS = 0;
  SLIM_BLOCKS = 1;
  FILL_BAR = 2;

type
  ARFactory = interface
    // returns R_OK if object is available
    function GetAPIObject(objectID: integer; var pObject: Pointer): integer; stdcall;
  end;

type
  ARComponent = interface
    procedure SetText(Text: RString); stdcall;
    procedure SetSize(Width, Height: integer); stdcall;
    procedure SetPosition(x, y: integer); stdcall;
    procedure SetVisible(Visible: R_BOOL); stdcall;
    procedure SetEnable(enable: R_BOOL); stdcall;
    procedure GetText(var Text: RString); stdcall;
    function GetWidth: integer; stdcall;
    function GetHeight: integer; stdcall;
    function GetPosX: integer; stdcall;
    function GetPosY: integer; stdcall;
    function GetID: integer; stdcall;
    function IsVisible: R_BOOL; stdcall;
    function IsEnabled: R_BOOL; stdcall;
  end;

type
  ARButtonListener = interface
    procedure OnButtonPress(buttonID: integer); stdcall;
  end;

type
  ARButton = interface(ARComponent)
    procedure SetListener(listener: ARButtonListener); stdcall;
  end;

type
  ARLabel = interface(ARComponent)
    procedure SetTextColor(textColor: COLORREF); stdcall;
    procedure EnableAutoSize(enable: R_BOOL); stdcall;
  end;

type
  ARCheckBoxListener = interface
    procedure OnCheckBoxStateChange(checkBoxID: integer); stdcall;
  end;

type
  ARCheckBox = interface(ARComponent)
    procedure SetListener(listener: ARCheckBoxListener); stdcall;
    function IsChecked: R_BOOL; stdcall;
    procedure SetCheckedState(state: R_BOOL); stdcall;
    procedure EnableAutoSize(enable: R_BOOL); stdcall;
  end;

type
  ARSliderListener = interface
    procedure OnSliderChange(sliderID: integer); stdcall;
  end;

type
  ARSlider = interface(ARComponent)
    procedure SetListener(listener: ARSliderListener); stdcall;
    procedure SetValue(Value: integer); stdcall;
    function GetValue: integer; stdcall;
  end;

type
  ARComboBoxListener = interface
    procedure OnComboBoxItemSelect(comboBoxID: integer); stdcall;
  end;

type
  ARComboBox = interface(ARComponent)
    procedure AddItem(Text: RString); stdcall;
    procedure UpdateItem(index: integer; Text: RString); stdcall;
    procedure GetItemText(index: integer; var ppText: RString); stdcall;
    procedure RemoveItem(index: integer); stdcall;
    procedure RemoveAll; stdcall;
    function GetSelectedItemIndex: integer; stdcall;
    procedure SetSelectedItem(index: integer); stdcall;
    function GetItemCount: integer; stdcall;
    procedure SetListener(listener: ARComboBoxListener); stdcall;
  end;

type
  ARProgressBar = interface(ARComponent)
    procedure SetValue(Value: integer); stdcall;
    function GetValue: integer; stdcall;
    procedure SetColor(color: COLORREF); stdcall;
  end;

// quick way to prototype a plugin
type
  UserInterfaceAPI = interface
    procedure CreateButton(parent: HWND; var ppButton: ARButton); stdcall;
    procedure CreateLabel(parent: HWND; isHeader: R_BOOL; var ppLabel: ARLabel); stdcall;
    procedure CreateCheckBox(parent: HWND; var ppCheckBox: ARCheckBox); stdcall;
    procedure CreateSlider(parent: HWND; isVertical: R_BOOL; var ppSlider: ARSlider); stdcall;
    procedure CreateComboBox(parent: HWND; var ppComboBox: ARComboBox); stdcall;
    procedure CreateProgressBar(parent: HWND; style: integer; var ppProgressBar: ARProgressBar); stdcall;
    function GetPluginWindow(containerWnd: HWND): HWND; stdcall;
    function GetApplicationWindow : HWND; stdcall;
    procedure SetContainerSize(parent: HWND; width: integer; height: integer); stdcall;
    procedure UpdatePluginName; stdcall;
  end;

implementation

end.
