
// Helper class to create audio effect plugins quickly!!!

unit ARPluginHelperUnit;

interface

uses
  Classes, Windows, SysUtils,
  RObjBaseUnit, RAudioEffectUnit, ARObjectsUnit;

type
  ARPluginHelper = class(TInterfacedObject, RAudioEffect)
  protected
    factory: ARFactory;
    interfaceAPI: UserInterfaceAPI;
  public
    function Initialize(hInstance: HINST; hostID: integer; hostParam: Pointer): integer; virtual; stdcall;
    procedure GetEffectName(var ppText: RString); virtual; stdcall;
    function GetEffectID: integer; virtual; stdcall;
    function GetEffectVersion: integer; virtual; stdcall;
    function SetChannelCount(inputChannelCount, outputChannelCount: integer): integer; virtual; stdcall;
    procedure SetSampleRate(srate: single); virtual; stdcall;
    procedure SetBlockSize(blockSize: integer); virtual; stdcall;
    procedure CreateEditor(container: HWND); virtual; stdcall;
    procedure GetEditorSize(Width, Height: PInteger); virtual; stdcall;
    procedure DestroyEditor; virtual; stdcall;
    procedure LoadState(data: Pointer; size: integer); virtual; stdcall;
    procedure StartProcess; virtual; stdcall;
    procedure ProcessAudio(inputs, outputs: PPSingle; frameCount: integer); virtual; stdcall;
    procedure StopProcess; virtual; stdcall;
    function SaveState(var data: Pointer): integer; virtual; stdcall;
    procedure Shutdown; virtual; stdcall;
  end;

implementation

function ARPluginHelper.Initialize(hInstance: HINST; hostID: integer; hostParam: Pointer): integer; stdcall;
  begin
    if hostID = AUDIO_REPEATER_PRO_HOST_ID then
      begin
        factory := ARFactory(hostParam);
        if factory.GetAPIObject(AUDIO_REPEATER_PRO_GUI_API, Pointer(interfaceAPI)) = R_OK then
          Result := 1
        else
          Result := 0;
      end
    else Result := 0;
  end;

  procedure ARPluginHelper.GetEffectName(var ppText: RString); stdcall;
  begin
    ppText := MakeRString('Unknown');
  end;

  function ARPluginHelper.GetEffectID: integer; stdcall;
  begin
    Result := 1000; // use your own id
  end;

  function ARPluginHelper.GetEffectVersion: integer; stdcall;
  begin
    Result := 1;
  end;

  function ARPluginHelper.SetChannelCount(inputChannelCount, outputChannelCount: integer): integer; stdcall;
  begin
    if (inputChannelCount = 2) and (outputChannelCount = 2) then
      Result := 1
    else
      Result := 0;
  end;

  procedure ARPluginHelper.SetSampleRate(srate: single); stdcall;
  begin

  end;

  procedure ARPluginHelper.SetBlockSize(blockSize: integer); stdcall;
  begin

  end;

  procedure ARPluginHelper.GetEditorSize(Width, Height: PInteger); stdcall;
  begin
    Width^ := 200;
    Height^ := 100;
  end;

  procedure ARPluginHelper.CreateEditor(container: HWND); stdcall;
  begin

  end;

  procedure ARPluginHelper.DestroyEditor; stdcall;
  begin

  end;

  procedure ARPluginHelper.LoadState(Data: Pointer; size: integer); stdcall;
  begin

  end;

  procedure ARPluginHelper.StartProcess; stdcall;
  begin

  end;

  procedure ARPluginHelper.ProcessAudio(inputs, outputs: PPSingle; frameCount: integer); stdcall;
  var
    index: integer;
    leftSample, rightSample: single;
    leftInBuffer, rightInBuffer: PSingle;
    leftOutBuffer, rightOutBuffer: PSingle;
  begin
    leftInBuffer := inputs^;
    Inc(inputs);
    rightInBuffer := inputs^;

    leftOutBuffer := outputs^;
    Inc(outputs);
    rightOutBuffer := outputs^;

    for index := 0 to frameCount - 1 do
    begin
      leftSample := leftInBuffer^;
      rightSample := rightInBuffer^;

      leftOutBuffer^ := leftSample;
      rightOutBuffer^ := rightSample;

      // move on to the next sample
      Inc(leftInBuffer);
      Inc(rightInBuffer);
      Inc(leftOutBuffer);
      Inc(rightOutBuffer);
    end;
  end;

  procedure ARPluginHelper.StopProcess; stdcall;
  begin

  end;

  function ARPluginHelper.SaveState(var Data: Pointer): integer; stdcall;
  begin
    Result := 0;
  end;

  procedure ARPluginHelper.Shutdown; stdcall;
  begin
    factory := nil;
    interfaceAPI := nil;
  end;

end.

