
// (c) 2018 CrownSoft

unit ARRemoteUnit;

interface

uses
  Windows,
  messages;

const
  AR_MAIN_WINDOW_CLASS_32 = 'Audio Repeater Pro Class - 32';
  AR_MAIN_WINDOW_CLASS_64 = 'Audio Repeater Pro Class - 64';
  AR_MAIN_WINDOW_TEXT = 'Audio Repeater Pro';

  LOAD_FX_CHAIN = 6678;
  SET_BYPASS_FX_ENABLE = 6679;
  SET_PROCESSING_ENABLE = 6680;
  SET_STREAM_GAIN = 6681;
  REPEATER_START = 6682;
  REPEATER_STOP = 6683;
  IS_BYPASS_FX_ENABLED = 6684;
  IS_PROCESSING_ENABLED = 6685;
  GET_STREAM_GAIN = 6686;
  IS_REPEATER_STARTED = 6687;

type
  ARRemote = class
  protected
    window: HWND;
  public
    function Connect(is32Bit: Boolean) : Boolean;
    procedure LoadFxChain(fileName: WideString);
    procedure SetEnableFxBypass(enable: Boolean);
    procedure SetEnableProcessing(enable: Boolean);
    procedure SetStreamGain(gain: Integer);
    procedure StartRepeater;
    procedure StopRepeater;
    function IsBypassFxEnabled() : Boolean;
    function IsProcessingEnabled() : Boolean;
    function IsRepeaterStarted() : Boolean;
    function GetStreamGain(): Integer;
  end;

implementation

function ARRemote.Connect(is32Bit: Boolean) : Boolean;
begin
   if is32Bit then
    window := FindWindow(AR_MAIN_WINDOW_CLASS_32, AR_MAIN_WINDOW_TEXT)
   else
    window := FindWindow(AR_MAIN_WINDOW_CLASS_64, AR_MAIN_WINDOW_TEXT);

   if window <> 0 then
    Result := true
   else
    Result := false;
end;

procedure ARRemote.LoadFxChain(fileName: WideString);
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := LOAD_FX_CHAIN;
  copyDataStruct.cbData := (1 + Length(fileName)) * 2;
  copyDataStruct.lpData := PWideChar(fileName);

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

procedure ARRemote.SetEnableFxBypass(enable: Boolean);
var
  copyDataStruct: TCopyDataStruct;
  value: Integer;
begin
  if enable then
    value := 1
  else
    value := 0;

  copyDataStruct.dwData := SET_BYPASS_FX_ENABLE;
  copyDataStruct.cbData := 4;
  copyDataStruct.lpData := @value;

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

procedure ARRemote.SetEnableProcessing(enable: Boolean);
var
  copyDataStruct: TCopyDataStruct;
  value: Integer;
begin
  if enable then
    value := 1
  else
    value := 0;

  copyDataStruct.dwData := SET_PROCESSING_ENABLE;
  copyDataStruct.cbData := 4;
  copyDataStruct.lpData := @value;

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

procedure ARRemote.SetStreamGain(gain: Integer);
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := SET_STREAM_GAIN;
  copyDataStruct.cbData := 4;
  copyDataStruct.lpData := @gain;

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

procedure ARRemote.StartRepeater;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := REPEATER_START;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

procedure ARRemote.StopRepeater;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := REPEATER_STOP;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

function ARRemote.IsBypassFxEnabled() : Boolean;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := IS_BYPASS_FX_ENABLED;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  Result := (SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct)) = 1);
end;

function ARRemote.IsProcessingEnabled() : Boolean;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := IS_PROCESSING_ENABLED;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  Result := (SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct)) = 1);
end;

function ARRemote.IsRepeaterStarted() : Boolean;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := IS_REPEATER_STARTED;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  Result := (SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct)) = 1);
end;

function ARRemote.GetStreamGain(): Integer;
var
  copyDataStruct: TCopyDataStruct;
begin
  copyDataStruct.dwData := GET_STREAM_GAIN;
  copyDataStruct.cbData := 0;
  copyDataStruct.lpData := nil;

  Result := SendMessage(window, WM_COPYDATA, 0, LPARAM(@copyDataStruct));
end;

end.
 