library ExamplePlugin2;

uses
  Classes,
  Windows,
  RAudioEffectUnit in '../../Common/RAudioEffectUnit.pas',
  RObjBaseUnit in '../../Common/RObjBaseUnit.pas',
  ARObjectsUnit in '../../Common/ARObjectsUnit.pas',
  ARPluginHelperUnit in '../../Common/ARPluginHelperUnit.pas';

{$R *.res}

type
  EffectState = record
    gain: integer;
  end;

  PEffectState = ^EffectState;

  TExamplePlugin2 = class(ARPluginHelper, ARSliderListener)
  public
    state: EffectState;
    slider: ARSlider;
    lblSlider: ARLabel;
    effectWindow: HWND;
    procedure GetEffectName(var ppText: RString); override; stdcall;
    function GetEffectID: integer; override; stdcall;
    procedure GetEditorSize(Width, Height: PInteger); override; stdcall;
    procedure ProcessAudio(inputs, outputs: PPSingle; frameCount: integer);
      override; stdcall;
    procedure CreateEditor(container: HWND); override; stdcall;
    procedure DestroyEditor; override; stdcall;
    procedure LoadState(Data: Pointer; size: integer); override; stdcall;
    function SaveState(var Data: Pointer): integer; override; stdcall;
    constructor Create;
    destructor Destroy; override;
    procedure OnSliderChange(sliderID: integer); stdcall;
  end;

  constructor TExamplePlugin2.Create;
  begin
    inherited Create;

    effectWindow := 0;
    state.gain := 50;
  end;

  destructor TExamplePlugin2.Destroy;
  begin

    inherited;
  end;

  procedure TExamplePlugin2.GetEffectName(var ppText: RString); stdcall;
  begin
    ppText := MakeRString('Audio Gain');
  end;

  function TExamplePlugin2.GetEffectID: integer; stdcall;
  begin
    Result := 4455; // use your own id
  end;

  procedure TExamplePlugin2.GetEditorSize(Width, Height: PInteger); stdcall;
  begin
    Width^ := 200;
    Height^ := 100;
  end;

  procedure TExamplePlugin2.CreateEditor(container: HWND); stdcall;
  begin
    effectWindow := container;

    interfaceAPI.CreateSlider(container, R_FALSE, slider);
    slider.SetPosition(40, 40);
    slider.SetValue(50);
    slider.SetListener(Self);

    interfaceAPI.CreateLabel(container, R_FALSE, lblSlider);
    lblSlider.SetText(MakeRString('Gain'));
    lblSlider.SetPosition(85, 70);
  end;

  procedure TExamplePlugin2.DestroyEditor; stdcall;
  begin
    slider := nil; // release slider object!
    lblSlider := nil; // release label object!
    effectWindow := 0;
  end;

  procedure TExamplePlugin2.OnSliderChange(sliderID: integer); stdcall;
  begin
    if sliderID = slider.GetID() then
      state.gain := slider.GetValue();
  end;

  procedure TExamplePlugin2.LoadState(Data: Pointer; size: integer); stdcall;
  begin
    if size = sizeof(EffectState) then
    begin
      state.gain := PEffectState(Data)^.gain;
      if effectWindow <> 0 then
        slider.SetValue(state.gain);
    end;
  end;

  function TExamplePlugin2.SaveState(var Data: Pointer): integer; stdcall;
  begin
    Data := @state;
    Result := sizeof(EffectState);
  end;

  procedure TExamplePlugin2.ProcessAudio(inputs, outputs: PPSingle;
    frameCount: integer); stdcall;
  var
    index: integer;
    leftSample, rightSample: single;
    leftInBuffer, rightInBuffer: PSingle;
    leftOutBuffer, rightOutBuffer: PSingle;
    mul: single;
  begin
    leftInBuffer := inputs^;
    Inc(inputs);
    rightInBuffer := inputs^;

    leftOutBuffer := outputs^;
    Inc(outputs);
    rightOutBuffer := outputs^;

    mul := (state.gain / 100.0) * 2;

    for index := 0 to frameCount - 1 do
    begin
      leftSample := leftInBuffer^;
      rightSample := rightInBuffer^;

      leftSample := leftSample * mul;
      rightSample := rightSample * mul;

      leftOutBuffer^ := leftSample;
      rightOutBuffer^ := rightSample;

      // move on to the next sample
      Inc(leftInBuffer);
      Inc(rightInBuffer);
      Inc(leftOutBuffer);
      Inc(rightOutBuffer);
    end;
  end;


  procedure GetEffect(var effect: RAudioEffect); stdcall;
  begin
    effect := TExamplePlugin2.Create;
  end;

exports
  GetEffect;

begin
end.
