
 // uses VCL Form as an editor.

library ExamplePlugin3;

uses
  Classes,
  Windows,
  RAudioEffectUnit in '..\..\Common\RAudioEffectUnit.pas',
  RObjBaseUnit in '..\..\Common\RObjBaseUnit.pas',
  ARObjectsUnit in '..\..\Common\ARObjectsUnit.pas',
  ARPluginHelperUnit in '..\..\Common\ARPluginHelperUnit.pas',
  Editor in 'Editor.pas' {Form1},
  EffectStateUnit in 'EffectStateUnit.pas';

{$R *.res}

type
  TExamplePlugin3 = class(ARPluginHelper)
  public
    effectWindow: HWND;
    editor: TForm1;
    state: EffectState;
    procedure GetEffectName(var ppText: RString); override; stdcall;
    function GetEffectID: integer; override; stdcall;
    procedure GetEditorSize(Width, Height: PInteger); override; stdcall;
    procedure ProcessAudio(inputs, outputs: PPSingle; frameCount: integer); override; stdcall;
    procedure CreateEditor(container: HWND); override; stdcall;
    procedure DestroyEditor; override; stdcall;
    procedure LoadState(data: Pointer; size: integer); override; stdcall;
    function SaveState(var data: Pointer): integer; override; stdcall;
    constructor Create;
    destructor Destroy; override;
  end;

  constructor TExamplePlugin3.Create;
  begin
    inherited Create;
    state.gain := 50;
  end;

  destructor TExamplePlugin3.Destroy;
  begin

    inherited;
  end;

  procedure TExamplePlugin3.GetEffectName(var ppText: RString); stdcall;
  begin
    ppText := MakeRString('Audio Gain');
  end;

  function TExamplePlugin3.GetEffectID: integer; stdcall;
  begin
    Result := 4455; // use your own id
  end;

  procedure TExamplePlugin3.GetEditorSize(Width, Height: PInteger); stdcall;
  begin
    Width^ := 213;
    Height^ := 108;
  end;

  procedure TExamplePlugin3.CreateEditor(container: HWND); stdcall;
  begin
		effectWindow := container;

    editor := TForm1.Create(nil);
    editor.ParentWindow := effectWindow;
    editor.state := @state;
    editor.Visible := True;
  end;

  procedure TExamplePlugin3.DestroyEditor; stdcall;
  begin
    editor.Destroy();
    effectWindow := 0;
  end;

  procedure TExamplePlugin3.LoadState(data: Pointer; size: integer); stdcall;
  begin
      if size = sizeof(EffectState) then
      begin
        state.gain := PEffectState(data)^.gain;
        if effectWindow <> 0 then
          editor.TrackBar1.Position := state.gain;
      end;
  end;

  function TExamplePlugin3.SaveState(var data: Pointer): integer; stdcall;
  begin
    data := @state;
    Result := sizeof(EffectState);
  end;

  procedure TExamplePlugin3.ProcessAudio(inputs, outputs: PPSingle; frameCount: integer); stdcall;
  var
    index: integer;
    leftSample, rightSample: single;
    leftInBuffer, rightInBuffer: PSingle;
    leftOutBuffer, rightOutBuffer: PSingle;
    mul: Single;
  begin
    leftInBuffer := inputs^;
    Inc(inputs);
    rightInBuffer := inputs^;

    leftOutBuffer := outputs^;
    Inc(outputs);
    rightOutBuffer := outputs^;

    mul := (state.gain / 100.0) * 2;

    for index := 0 to frameCount - 1 do
    begin
      leftSample := leftInBuffer^;
      rightSample := rightInBuffer^;

      leftSample := leftSample * mul;
      rightSample := rightSample * mul;

      leftOutBuffer^ := leftSample;
      rightOutBuffer^ := rightSample;

      // move on to the next sample
      Inc(leftInBuffer);
      Inc(rightInBuffer);
      Inc(leftOutBuffer);
      Inc(rightOutBuffer);
    end;
  end;


procedure GetEffect(var effect: RAudioEffect); stdcall;
  begin
    effect := TExamplePlugin3.Create;
  end;

exports
  GetEffect;

begin
end.
