
// Creates a window using standard gui service. Also adds an entry to the plugin menu.

#include "../../include/ARGeneralPlugin.h"
#include "../../include/ARStdServices.h"

// {102DBD26-9528-49B5-A3AE-C1DEE91F595F}
RGUID pluginID = { 0x102dbd26, 0x9528, 0x49b5, { 0xa3, 0xae, 0xc1, 0xde, 0xe9, 0x1f, 0x59, 0x5f } };

class ExamplePlugin2 : public ARGeneralPlugin, public TWindowListener, public ARMenuItemListener
{
	TWindow* pluginWindow;

public:

	int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam) override
	{
		if (ARGeneralPlugin::Initialize(hInstance, hostID, hostParam) == 0)
			return 0;

		pluginWindow = NULL;
		return 1;
	}

	void STDCALL Enable() override
	{
		this->CreatePluginWindow();
		systemAPI->AddToPluginMenu(SRSTR(L"Example Plugin 2"), this);
	}

	void STDCALL Disable() override
	{
		if (pluginWindow)
			this->DestroyPluginWindow();

		systemAPI->RemoveFromPluginMenu(this);
	}

	void STDCALL OnWindowMessage(TWindow* window, UINT message, WPARAM wParam, LPARAM lParam) override
	{
		if (window == pluginWindow)
		{
			if (message == WM_CLOSE)
				this->DestroyPluginWindow();
		}
	}

	void STDCALL OnMenuPress(int itemID) override
	{
		if (pluginWindow == NULL)
			this->CreatePluginWindow();
	}

	void CreatePluginWindow()
	{
		ARServiceFactory* stdServiceFactory;
		if (systemAPI->GetServiceFactory(&AR_STD_SERVICES_FACTORY_ID, &stdServiceFactory))
		{
			ARGUIService* guiService;
			if (stdServiceFactory->GetService(AR_GUI_SERVICE_ID, (void**)&guiService))
			{
				guiService->CreateHostWindow(TWindowTypes::FIXED_SIZE_WINDOW, &pluginWindow);

				pluginWindow->SetText(SRSTR(L"My Plugin Window"));
				pluginWindow->CenterScreen(0);
				pluginWindow->SetListener(this);
				pluginWindow->SetVisible(1);

				guiService->Release();
			}

			stdServiceFactory->Release();
		}
	}

	void DestroyPluginWindow()
	{
		pluginWindow->SetVisible(0);
		pluginWindow->Release();

		pluginWindow = NULL;
	}

	void STDCALL GetID(RGUID** id) override
	{
		*id = &pluginID;
	}

	void STDCALL GetName(RString** ppText) override
	{
		*ppText = MakeRString(L"Example Plug-In 2");
	}

	void STDCALL GetAuthor(RString** ppText) override
	{
		*ppText = MakeRString(L"CrownSoft");
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_PLUGIN(ExamplePlugin2)
SELF_REGISTER_PLUGIN("{102DBD26-9528-49B5-A3AE-C1DEE91F595F}")