
// Uses ARGeneralPluginHelper class to create a window. It also automatically loads and saves the plugin state.

#include "../../include/ARGeneralPluginHelper.h"

// {A02B4DC8-5C84-4EAD-B2A6-D716671CD409}
RGUID pluginID = { 0xa02b4dc8, 0x5c84, 0x4ead, { 0xb2, 0xa6, 0xd7, 0x16, 0x67, 0x1c, 0xd4, 0x9 } };

struct MyPluginState
{
	// following members must exist in your struct. they are used to store window properties.
	int posX;
	int posY;
	bool visible;

	// custom members here...
	int sliderValue;
};

class ExamplePlugin3 : public ARGeneralPluginHelper<MyPluginState>, public TSliderListener
{
protected:
	TSlider* slider1;
	TLabel* label1;

public:
	int OnInitialize(int hostID, void* hostParam) override
	{
		// assign default values to custom members of the plugin state
		pluginState.sliderValue = 0;

		return 1;
	}

	void OnCreateUI(ARServiceFactory* stdServiceFactory, ARGUIService* guiService) override
	{
		pluginWindow->SetSize(400, 200);

		guiService->CreateLabel(pluginWindow, 0, &label1);
		label1->SetPosition(20, 60);
		label1->SetText(SRSTR(L"Green Label"));
		label1->SetTextColor(RGB(0, 255, 0));

		guiService->CreateSlider(pluginWindow, 0, &slider1);
		slider1->SetPosition(20, 20);
		slider1->SetListener(this);

		// value of the slider will be set within OnApplyPluginStateToUI method.
	}

	void OnDestroyUI() override
	{
		// you must release the created components. pluginWindow will be released by the helper class.

		label1->Release();
		slider1->Release();
	}

	void OnApplyPluginStateToUI() override
	{
		slider1->SetValue(pluginState.sliderValue);
	}

	void STDCALL OnSliderChange(TSlider* slider) override
	{
		if (slider == slider1)
		{
			pluginState.sliderValue = slider1->GetValue();
		}
	}

	void STDCALL GetID(RGUID** id) override
	{
		*id = &pluginID;
	}

	const wchar_t* GetPluginName() override
	{
		return L"Example Plugin 3";
	}

	const wchar_t* GetPluginAuthor() override
	{
		return L"CrownSoft";
	}

	const wchar_t* GetPluginWindowText() override
	{
		return L"My Window";
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_PLUGIN(ExamplePlugin3)
SELF_REGISTER_PLUGIN("{A02B4DC8-5C84-4EAD-B2A6-D716671CD409}")