
// Uses ARGeneralPluginHelper class to host a native dialog.

#include "../../include/ARGeneralPluginHelper.h"
#include "./VisualStudio/resource.h"

// {F1DAB3B8-1F68-4735-8292-27343D19A30F}
RGUID pluginID = { 0xf1dab3b8, 0x1f68, 0x4735, { 0x82, 0x92, 0x27, 0x34, 0x3d, 0x19, 0xa3, 0xf } };


struct MyPluginState
{
	// following members must exist in your struct. they are used to store window properties.
	int posX;
	int posY;
	bool visible;

	// custom members here...

};

class ExamplePlugin4 : public ARGeneralPluginHelper<MyPluginState>
{
protected:
	HWND hwndDialog;

public:

	void OnCreateUI(ARServiceFactory* stdServiceFactory, ARGUIService* guiService) override
	{
		pluginWindow->SetSize(400, 200);

		// parent is the container
		hwndDialog = ::CreateDialogParamW(hInstance, MAKEINTRESOURCEW(IDD_DIALOG1),
			pluginWindow->GetContainerHandle(), ExamplePlugin4::DialogProc, (LPARAM)this);

		::ShowWindow(hwndDialog, SW_SHOW);
	}

	void OnDestroyUI() override
	{
		::DestroyWindow(hwndDialog);
	}

	INT_PTR HandleMessage(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
		switch (message)
		{
		case WM_INITDIALOG:
			return (INT_PTR)TRUE;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDC_BUTTON1:
				systemAPI->ShowNotification(SRSTR(L"Audio Repeater Pro"), SRSTR(L"Hello from example plugin 4!!!"));
				return TRUE;
			}
			return FALSE;

		default: return FALSE;
		}
	}

	static INT_PTR CALLBACK DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
		ExamplePlugin4* plugin;
		if (message == WM_INITDIALOG)
		{
			plugin = (ExamplePlugin4*)lParam;
			plugin->hwndDialog = hDlg;
			::SetWindowLongPtrW(hDlg, GWLP_USERDATA, (LONG_PTR)plugin);
			return plugin->HandleMessage(hDlg, message, wParam, lParam);
		}

		LONG_PTR ptr = ::GetWindowLongPtrW(hDlg, GWLP_USERDATA);
		plugin = (ExamplePlugin4*)ptr;

		if (plugin)
			return plugin->HandleMessage(hDlg, message, wParam, lParam);

		return FALSE;
	}

	void STDCALL GetID(RGUID** id) override
	{
		*id = &pluginID;
	}

	const wchar_t* GetPluginName() override
	{
		return L"Example Plugin 4";
	}

	const wchar_t* GetPluginAuthor() override
	{
		return L"CrownSoft";
	}

	const wchar_t* GetPluginWindowText() override
	{
		return L"My Window";
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_PLUGIN(ExamplePlugin4)
SELF_REGISTER_PLUGIN("{F1DAB3B8-1F68-4735-8292-27343D19A30F}")