
// Creates a native dialog. Does not use standard gui service or helper class.

#include "../../include/ARGeneralPlugin.h"
#include "./VisualStudio/resource.h"

// {1312B8E6-3B22-4517-839F-3600930EAF74}
RGUID pluginID = { 0x1312b8e6, 0x3b22, 0x4517, { 0x83, 0x9f, 0x36, 0x0, 0x93, 0xe, 0xaf, 0x74 } };

class ExamplePlugin5 : public ARGeneralPlugin, public ARMenuItemListener
{
protected:
	HWND hwndDialog;

public:

	void STDCALL Enable() override
	{
		systemAPI->AddToPluginMenu(SRSTR(L"Example Plugin 5"), this);

		HWND parentWindow;
		systemAPI->GetApplicationWindow(&parentWindow);

		// parent window is application window.
		hwndDialog = ::CreateDialogParamW(hInstance, MAKEINTRESOURCEW(IDD_DIALOG1), parentWindow, ExamplePlugin5::DialogProc, (LPARAM)this);

		::ShowWindow(hwndDialog, SW_SHOW);
	}

	void STDCALL Disable() override
	{
		systemAPI->RemoveFromPluginMenu(this);
		::DestroyWindow(hwndDialog);
	}

	void STDCALL OnMenuPress(int itemID) override
	{
		::ShowWindow(hwndDialog, SW_SHOW);
		::BringWindowToTop(hwndDialog);
	}

	INT_PTR HandleMessage(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
		switch (message)
		{
		case WM_INITDIALOG:
			return (INT_PTR)TRUE;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDC_BUTTON1:
				systemAPI->ShowNotification(SRSTR(L"Audio Repeater Pro"), SRSTR(L"Hello from example plugin 5!!!"));
				return TRUE;
			}
			return FALSE;
		case WM_CLOSE:
			::ShowWindow(hDlg, SW_HIDE);
			return FALSE;

		default: return FALSE;
		}
	}

	static INT_PTR CALLBACK DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
		ExamplePlugin5* plugin;
		if (message == WM_INITDIALOG)
		{
			plugin = (ExamplePlugin5*)lParam;
			plugin->hwndDialog = hDlg;
			::SetWindowLongPtrW(hDlg, GWLP_USERDATA, (LONG_PTR)plugin);
			return plugin->HandleMessage(hDlg, message, wParam, lParam);
		}

		LONG_PTR ptr = ::GetWindowLongPtrW(hDlg, GWLP_USERDATA);
		plugin = (ExamplePlugin5*)ptr;

		if (plugin)
			return plugin->HandleMessage(hDlg, message, wParam, lParam);

		return FALSE;
	}

	void STDCALL GetID(RGUID** id) override
	{
		*id = &pluginID;
	}

	void STDCALL GetName(RString** ppText) override
	{
		*ppText = MakeRString(L"Example Plug-In 5");
	}

	void STDCALL GetAuthor(RString** ppText) override
	{
		*ppText = MakeRString(L"CrownSoft");
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_PLUGIN(ExamplePlugin5)
SELF_REGISTER_PLUGIN("{1312B8E6-3B22-4517-839F-3600930EAF74}")