
// Register a simple service which can be consumed by the other plugins.

#include "../../include/ARGeneralPlugin.h"
#include "ExampleService.h"

// {0E4993D7-AA59-4039-BE35-39623EE0CDD1}
RGUID pluginID = { 0xe4993d7, 0xaa59, 0x4039, { 0xbe, 0x35, 0x39, 0x62, 0x3e, 0xe0, 0xcd, 0xd1 } };

class CExampleService : public CRUnknown, public ExampleService
{
public:
	int STDCALL Calculate(int x, int y)
	{
		return x + y;
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

class ExampleServiceFactory : public CRUnknown, public ARServiceFactory
{
public:
	void STDCALL GetFactoryName(RString** ppText)
	{
		*ppText = MakeRString(L"Example Service Factory");
	}

	int STDCALL GetService(int serviceID, void** pObject)
	{
		if (serviceID == EXAMPLE_SERVICE_ID)
		{
			*pObject = static_cast<ExampleService*>(new CExampleService());
			return 1;
		}

		*pObject = NULL;
		return 0;
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

class ExamplePlugin6 : public ARGeneralPlugin
{
protected:
	ExampleServiceFactory* exampleFactory;

public:

	int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam) override
	{
		if (ARGeneralPlugin::Initialize(hInstance, hostID, hostParam) == 0)
				return 0;

		exampleFactory = new ExampleServiceFactory();
		systemAPI->RegisterServiceFactory(&exampleServiceFactoryID, exampleFactory);

		return 1;
	}

	void STDCALL Free(int initOK) override
	{
		if (!initOK)
			return;

		systemAPI->UnregisterServiceFactory(&exampleServiceFactoryID);
		exampleFactory->Release();
	}

	void STDCALL GetID(RGUID** id) override
	{
		*id = &pluginID;
	}

	void STDCALL GetName(RString** ppText) override
	{
		*ppText = MakeRString(L"Example Plug-In 6");
	}

	void STDCALL GetAuthor(RString** ppText) override
	{
		*ppText = MakeRString(L"CrownSoft");
	}

	IMPLEMENTS_RUNKNOWN_METHODS
};

EXPORT_PLUGIN(ExamplePlugin6)
SELF_REGISTER_PLUGIN("{0E4993D7-AA59-4039-BE35-39623EE0CDD1}")