
#ifndef _AR_STD_SERVICES_H_
#define _AR_STD_SERVICES_H_

#include "ARGenInterfaces.h"

// {37336624-CB91-49D5-8C79-709A460A0ECA}
RGUID AR_STD_SERVICES_FACTORY_ID = { 0x37336624, 0xcb91, 0x49d5, { 0x8c, 0x79, 0x70, 0x9a, 0x46, 0xa, 0xe, 0xca } };

// ================= Standard GUI Services =========================

#define AR_GUI_SERVICE_V1	1000
#define AR_GUI_SERVICE_V2	1001

class TComponent : public RUnknown
{
public:
	virtual void STDCALL SetText(RString* text) = 0;
	virtual void STDCALL SetSize(int width, int height) = 0;
	virtual void STDCALL SetPosition(int x, int y) = 0;
	virtual void STDCALL SetVisible(int visible) = 0;
	virtual void STDCALL SetEnable(int enable) = 0;

	virtual void STDCALL GetText(RString** ppText) = 0;
	virtual int STDCALL GetWidth() = 0;
	virtual int STDCALL GetHeight() = 0;
	virtual int STDCALL GetPosX() = 0;
	virtual int STDCALL GetPosY() = 0;

	virtual int STDCALL IsVisible() = 0;
	virtual int STDCALL IsEnabled() = 0;
};

class TButton;

class TButtonListener : public RUnknown
{
public:
	virtual void STDCALL OnButtonPress(TButton* button) = 0;
};

// note: if you called SetSize then SetText should be called to align the text correctly.
class TButton : public TComponent
{
public:
	virtual void STDCALL SetListener(TButtonListener* listener) = 0;
};

class TToolBarButton;

class TToolBarButtonListener : public RUnknown
{
public:
	virtual void STDCALL OnToolBarButtonPress(TToolBarButton* button) = 0;
};

class TToolBarButton : public TComponent
{
public:
	// icon size must be 16x16
	virtual void STDCALL SetIcon(HICON icon) = 0;
	virtual void STDCALL SetToolTip(RString* toolTipText) = 0;
	virtual void STDCALL SetListener(TToolBarButtonListener* listener) = 0;
};

class TLabel : public TComponent
{
public:
	virtual void STDCALL SetTextColor(COLORREF textColor) = 0;
	virtual void STDCALL EnableAutoResize(int enable) = 0;
};

class TCheckBox;

class TCheckBoxListener : public RUnknown
{
public:
	virtual void STDCALL OnCheckBoxStateChange(TCheckBox* checkBox) = 0;
};

class TCheckBox : public TComponent
{
public:
	virtual void STDCALL SetListener(TCheckBoxListener* listener) = 0;
	virtual int STDCALL IsChecked() = 0;
	virtual void STDCALL SetCheckedState(int state) = 0;
	virtual void STDCALL EnableAutoResize(int enable) = 0;
};

class TSlider;

class TSliderListener : public RUnknown
{
public:
	virtual void STDCALL OnSliderChange(TSlider* slider) = 0;
};

class TSlider : public TComponent
{
public:
	virtual void STDCALL SetListener(TSliderListener* listener) = 0;
	virtual void STDCALL SetValue(int value) = 0;
	virtual int STDCALL GetValue() = 0;
};

class TComboBox;

class TComboBoxListener : public RUnknown
{
public:
	virtual void STDCALL OnComboBoxItemSelect(TComboBox* comboBox) = 0;
};

class TComboBox : public TComponent
{
public:
	virtual void STDCALL AddItem(RString* text) = 0;
	virtual void STDCALL UpdateItem(int index, RString* text) = 0;
	virtual void STDCALL GetItemText(int index, RString** ppText) = 0;
	virtual void STDCALL RemoveItem(int index) = 0;
	virtual void STDCALL RemoveAll() = 0;
	virtual int STDCALL GetSelectedItemIndex() = 0;
	virtual void STDCALL SetSelectedItem(int index) = 0;
	virtual int STDCALL GetItemCount() = 0;
	virtual void STDCALL SetListener(TComboBoxListener* listener) = 0;
};

namespace TProgressBarStyles
{
	enum TProgressBarStyles
	{
		FAT_BLOCKS = 0,
		SLIM_BLOCKS = 1,
		FILL_BAR = 2
	};
};

class TProgressBar : public TComponent
{
public:
	virtual void STDCALL SetValue(int value) = 0;
	virtual int STDCALL GetValue() = 0;
	virtual void STDCALL SetColor(COLORREF color) = 0;
};


class TEditBox : public TComponent
{
public:
	virtual void STDCALL SetTextColor(COLORREF color) = 0;
};

class TPasswordBox : public TComponent
{
public:
	virtual void STDCALL SetTextColor(COLORREF color) = 0;
};

class TNumericField : public TComponent
{
public:
	virtual void STDCALL SetTextColor(COLORREF color) = 0;
};

class TWindow;

class TWindowListener : public RUnknown
{
public:
	virtual void STDCALL OnWindowMessage(TWindow* window, UINT message, WPARAM wParam, LPARAM lParam) = 0;
};

class TWindow : public TComponent
{
public:
	// ======= AR_GUI_SERVICE_V1 ============
	virtual HWND STDCALL GetContainerHandle() = 0;
	virtual HWND STDCALL GetHostWindowHandle() = 0;
	virtual void STDCALL CenterScreen(int useDefaultMonitor) = 0;
	// icon size must be 16x16
	virtual void STDCALL SetIcon(HICON windowIcon) = 0;
	virtual void STDCALL SetListener(TWindowListener* listener) = 0;

	// ======= AR_GUI_SERVICE_V2 ============
	virtual void STDCALL SetEnableCloseButton(int enable) = 0;
};

namespace TWindowTypes {
	enum TWindowTypes
	{
		FIXED_SIZE_WINDOW
	};
}

class ARGUIService : public RUnknown
{
public:
	virtual int STDCALL CreateHostWindow(int windowType, TWindow** ppWindow) = 0;
	virtual int STDCALL CreateButton(TWindow* window, TButton** ppButton) = 0;
	virtual int STDCALL CreateToolBarButton(TWindow* window, RString* toolTipText, TToolBarButton** ppToolBarButton) = 0;
	virtual int STDCALL CreateLabel(TWindow* window, int isBold, TLabel** ppLabel) = 0;
	virtual int STDCALL CreateCheckBox(TWindow* window, TCheckBox** ppCheckBox) = 0;
	virtual int STDCALL CreateSlider(TWindow* window, int isVertical, TSlider** ppSlider) = 0;
	virtual int STDCALL CreateComboBox(TWindow* window, TComboBox** ppComboBox) = 0;
	virtual int STDCALL CreateProgressBar(TWindow* window, int progressBarStyle, TProgressBar** ppProgressBar) = 0;
	virtual int STDCALL CreateEditBox(TWindow* window, TEditBox** ppEditBox) = 0;
	virtual int STDCALL CreateHLine(TWindow* window, TComponent** ppHLine) = 0;

	// ======= AR_GUI_SERVICE_V2 ============
	virtual int STDCALL CreatePasswordBox(TWindow* window, TPasswordBox** ppPasswordBox) = 0;
	virtual int STDCALL CreateNumericField(TWindow* window, TNumericField** ppNumericFiled) = 0;
};

// ==================================================================

#endif
