
// COM compatible generic plugin class which is intended to be used with CrownSoft products.
// (c) 2021 CrownSoft

#ifndef _RPLUGIN_H_
#define _RPLUGIN_H_

#include "RObjBase.h"

#ifdef _WIN64
	#define RPLUGIN_EXT L".rp64"
#else
	#define RPLUGIN_EXT L".rp32"
#endif


class RPlugin : public RUnknown
{
public:

	virtual void STDCALL GetName(RString** ppText) = 0;

	virtual void STDCALL GetAuthor(RString** ppText) = 0;

	virtual void STDCALL GetID(RGUID** id) = 0;

	virtual int STDCALL GetVersion() = 0;

	// returns 0 on failure, register services, cannot use other services
	virtual int STDCALL Initialize(HINSTANCE hInstance, int hostID, void* hostParam) = 0;

	// can use other services
	virtual void STDCALL LoadState(void* data, int size) = 0;

	// enable processing, create and show window, use other services, register on context menu
	virtual void STDCALL Enable() = 0;

	// disable processing, hide and destroy window, can use other services, unregister on context menu
	virtual void STDCALL Disable() = 0;

	// can use other services
	virtual int STDCALL SaveState(void** data) = 0;

	// stop using other services, release other sevices, but provide services to others, services which are depending on other services should not invoke those services.
	virtual void STDCALL Stop() = 0;

	// unregister services and free resources, cannot use other services
	virtual void STDCALL Free(int initOK) = 0;
};

typedef void (STDCALL* GetPluginFuncPtr)(RPlugin**);

#define EXPORT_PLUGIN(className) extern "C" __declspec(dllexport) void STDCALL GetPlugin(RPlugin** plugin) \
{ \
	*plugin = static_cast<RPlugin*>(new className()); \
}

#endif