
// ARGenInterfacesUnit (c) 2021 CrownSoft

unit ARGenInterfacesUnit;

interface

uses
  Windows,
  RObjBaseUnit;

const
  AUDIO_REPEATER_PRO_HOST_ID = 9980;
  AUDIO_REPEATER_PRO_SYSTEM_API_V1 = 2001;

type
  ARSystemAPIFactory = interface
    function GetSystemAPI(apiID: integer; var pObject: Pointer): integer; stdcall;
  end;

type
  ARServiceFactory = interface
    procedure GetFactoryName(var ppText : RString); stdcall;
    function GetService(serviceID: integer; var pObject: Pointer): integer; stdcall;
  end;

type
  ARAudioCallback = interface
    procedure OnAudioData(inputBuffer, outputBuffer: PPSingle; framesCount,byteCount,sampleRate : Integer); stdcall;
  end;

type
  ARMenuItemListener = interface
    procedure OnMenuPress(itemID : Integer); stdcall;
  end;

const
  AR_EVENT_DEVICE_STATE_CHANGE = 4000;
  AR_EVENT_DEVICE_LIST_RELOADED	= 4001;
  AR_EVENT_STREAM_START = 4002;
  AR_EVENT_STREAM_STOP = 4003;
  AR_CUSTOM_EVENT = 20000;
  AR_DEVICE_STATE_ACTIVE = 1;
  AR_DEVICE_STATE_DISABLED = 2;
  AR_DEVICE_STATE_NOTPRESENT = 4;
  AR_DEVICE_STATE_UNPLUGGED = 8;

type
  AREventListener = interface
    procedure OnAREvent(eventID, param1 : Integer; param2 : Pointer); stdcall;
  end;

const
  MSG_INFORMATION = 79;
  MSG_ERROR = 80;
  MSG_QUESTION = 23;
  MSG_WARNING = 78;
  MSG_BUTTON_OK = 0;
  MSG_BUTTON_YES_NO = 1;

type
  ARSystemAPI = interface
    function GetBuildID : Integer; stdcall;

    function GetInputDeviceCount : Integer; stdcall;
    function GetOutputDeviceCount : Integer; stdcall;

    function GetInputDeviceNameByIndex(index : Integer; var deviceName : RString) : Integer; stdcall;
    function GetInputDeviceIDByIndex(index : Integer; var deviceID : RString) : Integer; stdcall;
    function GetOutputDeviceNameByIndex(index : Integer; var deviceName : RString) : Integer; stdcall;
    function GetOutputDeviceIDByIndex(index : Integer; var deviceID : RString) : Integer; stdcall;

    procedure GetCurrentInputDeviceID(var deviceID : RString); stdcall;
    procedure GetCurrentInputDeviceName(var deviceName : RString); stdcall;
    procedure GetCurrentOutputDeviceID(var deviceID : RString); stdcall;
    procedure GetCurrentOutputDeviceName(var deviceName : RString); stdcall;

    function SetCurrentInputDevice(deviceID : RString) : Integer; stdcall;
    function SetCurrentOutputDevice(deviceID : RString) : Integer; stdcall;

    function GetDeviceNameByID(deviceID : RString; var deviceName : RString) : Integer; stdcall;
    function IsDeviceAvailable(deviceID : RString) : Integer; stdcall;

    function IsStreaming : Integer; stdcall;
    function IsApplicationMinimized : Integer; stdcall;
    procedure GetApplicationWindow(var windowHandle : HWND); stdcall;
    procedure MinimizeApplication; stdcall;
    procedure RestoreApplication; stdcall;

    function ShowMessage(icon : Integer; title, text : RString; buttons : Integer) : Integer; stdcall;
    procedure ShowNotification(title, text : RString); stdcall;
  
    procedure AddToPluginMenu(menuText : RString; listener : ARMenuItemListener); stdcall;
    procedure RemoveFromPluginMenu(listener : ARMenuItemListener); stdcall;

    function GetServiceFactory(factoryID : PGUID; var ppFactory : ARServiceFactory) : Integer; stdcall;
    procedure RegisterServiceFactory(factoryID : PGUID; pFactory : ARServiceFactory); stdcall;
    procedure UnregisterServiceFactory(factoryID : PGUID); stdcall;

    procedure AddAudioCallback(audioCallback : ARAudioCallback); stdcall;
    procedure RemoveAudioCallback(audioCallback : ARAudioCallback); stdcall;

    procedure AddEventListener(eventListener : AREventListener); stdcall;
    procedure RemoveEventListener(eventListener : AREventListener); stdcall;

    procedure ReloadDeviceList; stdcall;
    procedure StartStreamingByID(inputDeviceID, outputDeviceID : RString; silentMode : Integer); stdcall;
    procedure StartStreamingByName(inputDeviceName, outputDeviceName : RString; silentMode : Integer); stdcall;
    procedure StopStreaming; stdcall;
    procedure PostEvent(eventListener : AREventListener; eventID, param1 : Integer; param2 : Pointer); stdcall;
  end;

implementation

end.
