
// ARStdServicesUnit (c) 2021 CrownSoft

unit ARStdServicesUnit;

interface

uses
  Windows,
  RObjBaseUnit;

const
  AR_STD_SERVICES_FACTORY_ID : TGUID = '{37336624-CB91-49D5-8C79-709A460A0ECA}';
  AR_GUI_SERVICE_ID	= 1000;

  // ProgressBar styles
  FAT_BLOCKS = 0;
  SLIM_BLOCKS = 1;
  FILL_BAR = 2;

  // Window types
  FIXED_SIZE_WINDOW = 0;
  
type
  StdComponent = interface
    procedure SetText(Text: RString); stdcall;
    procedure SetSize(Width, Height: integer); stdcall;
    procedure SetPosition(x, y: integer); stdcall;
    procedure SetVisible(Visible: integer); stdcall;
    procedure SetEnable(enable: integer); stdcall;
    procedure GetText(var Text: RString); stdcall;
    function GetWidth: integer; stdcall;
    function GetHeight: integer; stdcall;
    function GetPosX: integer; stdcall;
    function GetPosY: integer; stdcall;
    function IsVisible: integer; stdcall;
    function IsEnabled: integer; stdcall;
  end;

  StdButton = interface; // Forward declaration.

  StdButtonListener = interface
    procedure OnButtonPress(button: StdButton); stdcall;
  end;

  StdButton = interface(StdComponent)
    procedure SetListener(listener: StdButtonListener); stdcall;
  end;

  StdToolBarButton = interface; // Forward declaration.

  StdToolBarButtonListener = interface
    procedure OnToolBarButtonPress(button: StdToolBarButton); stdcall;
  end;

  StdToolBarButton = interface(StdComponent)
    procedure SetIcon(icon: HICON); stdcall;
    procedure SetToolTip(toolTipText: RString); stdcall;
    procedure SetListener(listener: StdToolBarButtonListener); stdcall;
  end;

  StdLabel = interface(StdComponent)
    procedure SetTextColor(textColor: COLORREF); stdcall;
    procedure EnableAutoSize(enable: integer); stdcall;
  end;

  StdCheckBox = interface; // Forward declaration.

  StdCheckBoxListener = interface
    procedure OnCheckBoxStateChange(checkBox: StdCheckBox); stdcall;
  end;

  StdCheckBox = interface(StdComponent)
    procedure SetListener(listener: StdCheckBoxListener); stdcall;
    function IsChecked: integer; stdcall;
    procedure SetCheckedState(state: integer); stdcall;
    procedure EnableAutoSize(enable: integer); stdcall;
  end;

  StdSlider = interface; // Forward declaration.

  StdSliderListener = interface
    procedure OnSliderChange(slider: StdSlider); stdcall;
  end;

  StdSlider = interface(StdComponent)
    procedure SetListener(listener: StdSliderListener); stdcall;
    procedure SetValue(Value: integer); stdcall;
    function GetValue: integer; stdcall;
  end;

  StdComboBox = interface; // Forward declaration.

  StdComboBoxListener = interface
    procedure OnComboBoxItemSelect(comboBox: StdComboBox); stdcall;
  end;

  StdComboBox = interface(StdComponent)
    procedure AddItem(Text: RString); stdcall;
    procedure UpdateItem(index: integer; Text: RString); stdcall;
    procedure GetItemText(index: integer; var ppText: RString); stdcall;
    procedure RemoveItem(index: integer); stdcall;
    procedure RemoveAll; stdcall;
    function GetSelectedItemIndex: integer; stdcall;
    procedure SetSelectedItem(index: integer); stdcall;
    function GetItemCount: integer; stdcall;
    procedure SetListener(listener: StdComboBoxListener); stdcall;
  end;

  StdProgressBar = interface(StdComponent)
    procedure SetValue(Value: integer); stdcall;
    function GetValue: integer; stdcall;
    procedure SetColor(color: COLORREF); stdcall;
  end;

  StdEditBox = interface(StdComponent)
    procedure SetTextColor(color: COLORREF); stdcall;
  end;

  StdWindow = interface; // Forward declaration.

  StdWindowListener = interface
    procedure OnWindowMessage(window: StdWindow; message : UINT; wParam: WPARAM; lParam: LPARAM); stdcall;
  end;

  StdWindow = interface(StdComponent)
     function GetContainerHandle: HWND; stdcall;
     function GetHostWindowHandle: HWND; stdcall;
     procedure CenterScreen(useDefaultMonitor : integer); stdcall;
     procedure SetIcon(windowIcon: HICON); stdcall;
     procedure SetListener(listener: StdWindowListener); stdcall;
  end;

  ARGUIService = interface
    function CreateHostWindow(windowType: integer; var ppWindow: StdWindow): integer;  stdcall;
    function CreateButton(window: StdWindow; var ppButton: StdButton): integer;  stdcall;
    function CreateToolBarButton(window: StdWindow; toolTipText: RString; var ppToolBarButton: StdToolBarButton): integer;  stdcall;
    function CreateLabel(window: StdWindow; isBold: integer; var ppLabel: StdLabel): integer;  stdcall;
    function CreateCheckBox(window: StdWindow; var ppCheckBox: StdCheckBox): integer;  stdcall;
    function CreateSlider(window: StdWindow; isVertical: integer; var ppSlider: StdSlider): integer;  stdcall;
    function CreateComboBox(window: StdWindow; var ppComboBox: StdComboBox): integer;  stdcall;
    function CreateProgressBar(window: StdWindow; progressBarStyle: integer; var ppProgressBar: StdProgressBar): integer;  stdcall;
    function CreateEditBox(window: StdWindow; var ppEditBox: StdEditBox): integer;  stdcall;
    function CreateHLine(window: StdWindow; var ppHLine: StdComponent): integer;  stdcall;
  end;


implementation

end.
 