library ExamplePlugin2;

uses
  SysUtils,
  Classes,
  RObjBaseUnit in '../Common/RObjBaseUnit.pas',
  RPluginUnit in '../Common/RPluginUnit.pas',
  ARGeneralPluginUnit in '../Common/ARGeneralPluginUnit.pas',
  ARGenInterfacesUnit in '../Common/ARGenInterfacesUnit.pas',
  ARStdServicesUnit in '../Common/ARStdServicesUnit.pas';

{$R *.res}

const
   pluginID : TGUID = '{24DBCC66-72AA-4806-AE28-B55FC5B83FC8}';

type
  TExamplePlugin2 = class(ARGeneralPlugin, ARMenuItemListener, StdButtonListener)
  protected
    pluginWindow: StdWindow;
    button1: StdButton;
  public
    constructor Create;
    destructor Destroy; override;

    procedure GetID(var ppID : PGUID); override; stdcall;
    procedure GetName(var ppText : RString); override; stdcall;
    procedure GetAuthor(var ppText : RString); override; stdcall;

    procedure Enable; override; stdcall;
    procedure Disable; override; stdcall;

    procedure OnMenuPress(itemID : Integer); stdcall;
    procedure OnButtonPress(button: StdButton); stdcall;
  end;

  constructor TExamplePlugin2.Create;
  begin
    inherited Create;
    // put your code here...
  end;

  destructor TExamplePlugin2.Destroy;
  begin
    // put your code here...
    inherited;
  end;

  procedure TExamplePlugin2.Enable; stdcall;
  var
  stdServicesFactory: ARServiceFactory;
  guiService: ARGUIService;
  begin
    if systemAPI.GetServiceFactory(@AR_STD_SERVICES_FACTORY_ID, stdServicesFactory) = 1 then
    begin
      if stdServicesFactory.GetService(AR_GUI_SERVICE_ID, Pointer(guiService)) = 1 then
      begin
         guiService.CreateHostWindow(FIXED_SIZE_WINDOW, pluginWindow);
         pluginWindow.CenterScreen(0);
         pluginWindow.SetText(RSTR('Example Plug-In 2'));

         guiService.CreateButton(pluginWindow, button1);
         button1.SetPosition(20, 20);
         button1.SetText(RSTR('Press'));
         button1.SetListener(Self);

         pluginWindow.SetVisible(1);
         systemAPI.AddToPluginMenu(RSTR('Example Plug-In 2'), Self);

         guiService := nil; // explicitly release
      end;
      stdServicesFactory := nil;
    end;
  end;

  procedure TExamplePlugin2.Disable; stdcall;
  begin
      if pluginWindow <> nil then
      begin
        button1 := nil;
        pluginWindow := nil;
        systemAPI.RemoveFromPluginMenu(Self);
      end;
  end;

  procedure TExamplePlugin2.OnMenuPress(itemID : Integer); stdcall;
  begin
     pluginWindow.SetVisible(1);
  end;

  procedure TExamplePlugin2.OnButtonPress(button: StdButton); stdcall;
  var
  deviceName: RString;
  begin
     if button = button1 then
     begin
       systemAPI.GetCurrentInputDeviceName(deviceName);
       systemAPI.ShowNotification(RSTR('Example Plug-In 2'), deviceName);
     end; 
  end; 

  procedure TExamplePlugin2.GetName(var ppText : RString); stdcall;
  begin
     ppText := RSTR('Example Plug-In 2');
  end;

  procedure TExamplePlugin2.GetAuthor(var ppText : RString); stdcall;
  begin
     ppText := RSTR('CrownSoft');
  end;

  procedure TExamplePlugin2.GetID(var ppID : PGUID); stdcall;
  begin
    ppID := @pluginID;
  end;

procedure GetPlugin(var plugin: RPlugin); stdcall;
  begin
    plugin := TExamplePlugin2.Create;
  end;

exports
  GetPlugin;
  
begin
end.
 